#include "StdAfx.h"
#ifdef __USE_PROFILE_IMG_BY_P2P__
#include "GameTalkAPIGlobal.h"
#include "ProfileImageRecverP2P.h"
#include "UtilFunction.h"
#include "ProfileImageActor.h"
#include "MessengerActor.h"
#include "NpProtocolCodeDefine.h"
#include "ConvertUTF.h"
#include "GBStream.h"
#include "WASActor.h"


CProfileImageRecverP2P::CProfileImageRecverP2P(CProfileImageActor *pActor) : 
	m_pProfileImageActor(pActor), m_bWorking(false), m_bReady(false),
	m_pSession(NULL)
{
	m_tridDownloadFromWAS = 0;

	m_sData.dwCRC = 0;
}

CProfileImageRecverP2P::~CProfileImageRecverP2P(void)
{
	if(m_pSession)
		m_pSession->Delete();

	CWASActor *pWASActor = static_cast<CWASActor *>(m_pProfileImageActor->GetMessengerActor()->GetWASActor());
	if(pWASActor)
		pWASActor->UnRegisterObserverEx(this);
}


bool CProfileImageRecverP2P::RequestGetImage(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, DWORD dwUserData1, DWORD dwUserData2)
{
	if(m_bWorking)
		return true;

	m_bWorking = true;

	// Setting Info..
	m_sData.eSizeType = eSizeType;
	m_sData.strUserID = strUserID;

	m_dwUserData1 = dwUserData1;
	m_dwUserData2 = dwUserData2;

	do
	{
		if(m_pProfileImageActor->GetNpP2PMngr() == NULL)
			break;

		if(m_pSession)
			m_pSession->Delete();

		m_pSession = m_pProfileImageActor->GetNpP2PMngr()->CreateNpStreamSession(this);
		m_pSession->ConnectTcpSession(m_sData.strUserID.c_str());
		return true;
	} while(false);

	m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_Internal);
	
	// NEVER WRITE CODE BELOW THIS!!!
	// DELETED THIS OBJECT..
	return false;
}


// ڱ ̹ ޾ƿö .  WAS ϸ ٸ ̵ . ( buddyid Ķ͸ ߰ϸ )
bool CProfileImageRecverP2P::RequestGetImageFromWAS(void)
{
	do
	{
		CWASActor *pWASActor = static_cast<CWASActor *>(m_pProfileImageActor->GetMessengerActor()->GetWASActor());
		if(NULL == pWASActor)
			break;

		pWASActor->RegisterObserverEx(this);

		GT_PROFILE_IMAGE_DATA sData;
		m_sData.Convert(sData);

		if(pWASActor->RequestProfileImageProfile(false, sData, m_dwUserData1, m_dwUserData2, &m_tridDownloadFromWAS) == false)
			break;

		return true;
	}while(false);

	return false;
}


bool CProfileImageRecverP2P::CheckImage(void)
{
	if(m_sData.strFileName.empty())
		return false;

#if defined(WIN32)
	HBITMAP hBitmap = (HBITMAP)ImageUtil::LoadImage(m_sData.strFileName.c_str());
	if(hBitmap == NULL)
		return false;
	DeleteObject(hBitmap);
#endif

	DWORD dwCRC;
	if(CProfileImageActor::GetFileCRC(m_sData.strFileName.c_str(), &dwCRC) == 0)
		return false;

	////    о  CRC κ   CRC Ʋ !!
	if(m_sData.dwCRC != 0 && m_sData.dwCRC != dwCRC)
		return FALSE;

    m_bReady = true;
	return true;
}


bool CProfileImageRecverP2P::CopyFromTempFile(void)
{
	DWORD dwCRC = 0;
	DWORD dwSize = CProfileImageActor::GetFileCRC(m_strTempFile.c_str(), &dwCRC);
	if(dwSize == 0)
		return FALSE;

	if(m_sData.dwCRC != dwCRC)
		return FALSE;

	BOOL bRet = CopyFile(m_strTempFile.c_str(), m_sData.strFileName.c_str(), FALSE);

	DWORD dwError = GetLastError();

	//ERROR_SHARING_VIOLATION
	// 32 (0x20) The process cannot access the file because it is being used by another process.
	if(dwError == 32)
		bRet = true;

	DeleteFile(m_strTempFile.c_str());
	return bRet ? true : false;
}




//------------------------------------------------------------------------------------------------
//
// _INpStreamSessionEvents 
//
//------------------------------------------------------------------------------------------------

void CALLBACK CProfileImageRecverP2P::NpStreamSession_OnConnect(INpStreamSession *pSession, int nResultCode, int nErrorCode)
{
	if(nErrorCode == 0)
	{
		WORD tmp = m_sData.eSizeType;
		pSession->SendNpPacket_M6V2(REQ_PFIMG_CKSUM, 2, (LPCSTR)&tmp);
	} 
	else
	{
		if(!RequestGetImageFromWAS())
		{
			m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_WAS);

			// NEVER WRITE CODE BELOW THIS!!!
			// DELETED THIS OBJECT..
			return;
		}
	}
}


void CALLBACK CProfileImageRecverP2P::NpStreamSession_OnClose(INpStreamSession *pSession, int nSocketError, bool bRecvInvalidPacket)
{
	if(m_bReady == false)
	{
		pSession->CloseSession();
		if(!RequestGetImageFromWAS())
		{
			m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_WAS);
			// NEVER WRITE CODE BELOW THIS!!!
			// DELETED THIS OBJECT..
			return;
		}
	}
}


void CALLBACK CProfileImageRecverP2P::NpStreamSession_OnRecvPacket(INpStreamSession *pSession, INpPacket * pPacket)
{	
	switch(pPacket->M6V2()->usCMMD)
	{
	case RESPONSE(REQ_PFIMG_CKSUM):
		{
			nfc::CGBStream stream;
			DWORD dwCRC = 0;
			nfc::UTF8 szUTF8FileName[257];
			try
			{
				stream.UseExternalBuffer(pPacket->BodyData(), pPacket->BodyLength());
				dwCRC = stream.Read_U4();
				stream.Read_U8SS(szUTF8FileName);
			} 
			catch(nfc::CGBStreamException e)
			{
			}

			
			WSTRING strProfilePath = CProfileImageActor::GetProfileImageFilePathBySizeType(m_sData.strUserID.c_str(), m_sData.eSizeType);

			DWORD dwLatestCRC;
			if(CProfileImageActor::GetFileCRC(strProfilePath.c_str(), &dwLatestCRC) > 0)
			{
				if(dwCRC == dwLatestCRC)
				{
					// Ϸ!! (̹ ִ  ״ ̿!)
					m_sData.strFileName = strProfilePath;
					m_sData.dwCRC = dwCRC;

					pSession->CloseSession();
					m_bWorking = false;
					m_pProfileImageActor->OnProfileImageReady(this);
					return;
					// NEVER WRITE CODE BELOW THIS!!!
					// DELETED THIS OBJECT..
				}
			}

			if(m_bReady && m_sData.dwCRC == dwCRC)
			{
				// Ϸ!!
				pSession->CloseSession();
				m_bWorking = false;
				m_pProfileImageActor->OnProfileImageReady(this);
				return;
				// NEVER WRITE CODE BELOW THIS!!!
				// DELETED THIS OBJECT..
			} 
			else
			{
				USES_UTF8_CONVERSION

				// ̹  ..
				m_sData.strFileName = strProfilePath;//CProfileImageActor::GetProfileImageFilePath(m_sData.strUserID.c_str(), U82W((LPCSTR)szUTF8FileName));
				m_sData.dwCRC = dwCRC;
		
				//  Ͽ ϴ Ѵ.
				m_strTempFile = m_sData.strFileName + _T("_temp");
				m_hTempFile = CreateFile(m_strTempFile.c_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
				if(m_hTempFile == INVALID_HANDLE_VALUE)
				{
					pSession->CloseSession();
					if(!RequestGetImageFromWAS())
					{
						m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_WAS);
						// NEVER WRITE CODE BELOW THIS!!!
						// DELETED THIS OBJECT..
						return;
					}
				}
				BYTE nTmp = 0;
				pSession->SendNpPacket_M6V2(REQ_PFIMG_REQDN, 1, (LPCSTR)&nTmp);
			}
		}
		break;

	case RESPONSE(REQ_PFIMG_REQDN):
		{
			if(m_hTempFile == NULL || pPacket->BodyLength() == 0)
			{
				pSession->CloseSession();
				if(!RequestGetImageFromWAS())
				{
					m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_WAS);
					// NEVER WRITE CODE BELOW THIS!!!
					// DELETED THIS OBJECT..
					return;
				}
				return;
			}

			bool bLast = (pPacket->BodyData()[0] == 1);
			DWORD dwWritten;
			if(WriteFile(m_hTempFile, pPacket->BodyData() + 1, pPacket->BodyLength() - 1, &dwWritten, NULL) == FALSE)
			{
				CloseHandle(m_hTempFile);
				m_hTempFile = NULL;
				pSession->CloseSession();
				m_bWorking = false;
				m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_Disk);
				// NEVER WRITE CODE BELOW THIS!!!
				// DELETED THIS OBJECT..
				return;
			}

			if(bLast)
			{
				CloseHandle(m_hTempFile);
				m_hTempFile = NULL;
				if(CopyFromTempFile() && CheckImage())
				{
					m_bWorking = false;
					m_pProfileImageActor->OnProfileImageReady(this);
					return;
					// NEVER WRITE CODE BELOW THIS!!!
					// DELETED THIS OBJECT..
				}
				else
				{
					pSession->CloseSession();
					if(!RequestGetImageFromWAS())
					{
						m_pProfileImageActor->OnProfileImageError(this, wasError_NotRequest, piError_WAS);
						// NEVER WRITE CODE BELOW THIS!!!
						// DELETED THIS OBJECT..
						return;
					}
				}
			}
		}
		break;
	}
}


void CALLBACK CProfileImageRecverP2P::NpStreamSession_OnSendReady(INpStreamSession * /*pSession*/)
{
	// Nothing to do..
}	





//------------------------------------------------------------------------------------------------
//
// IGTWASActorEvent 
//
//------------------------------------------------------------------------------------------------

bool CProfileImageRecverP2P::OnGTWASActor_ResponseProfileImage(IGTWASActor *pActor, 
														   trid_t trid,
														   GTWASCallErrorCode eErrorCode, 
														   GTProfileImgError eProfileError, 
														   bool bUploadFile, 
														   const GT_PROFILE_IMAGE_DATA& sData,
                                                           DWORD dwUserData1,
                                                           DWORD dwUserData2)
{
	if(bUploadFile)
		return true;

	if(trid != m_tridDownloadFromWAS)
		return true;

	m_bWorking = false;

	if(eErrorCode == wasError_None && (eProfileError == piError_None || eProfileError == piError_NoImage))
	{
        m_pProfileImageActor->SaveCurrentProfileImageCRC(sData.strUserID, sData.dwCRC, sData.eSizeType); 

		// Save Result..
		m_sData.dwCRC = sData.dwCRC;
		m_sData.strFileName = sData.strFileName;

		m_pProfileImageActor->OnProfileImageReady(this);
		// NEVER WRITE CODE BELOW THIS!!!
		// DELETED THIS OBJECT..
		return true;
	}
	else
	{
		m_pProfileImageActor->OnProfileImageError(this, eErrorCode, piError_WAS);
		// NEVER WRITE CODE BELOW THIS!!!
		// DELETED THIS OBJECT..
		return true;
	}
    
	return true;
}
#endif